package com.emsoft.groovy

import java.lang.annotation.ElementType
import java.lang.annotation.Retention
import java.lang.annotation.RetentionPolicy
import java.lang.annotation.Target
import org.codehaus.groovy.transform.GroovyASTTransformationClass

@Retention(RetentionPolicy.SOURCE)
@Target(ElementType.METHOD)
@GroovyASTTransformationClass("com.emsoft.groovy.transform.StopwatchASTTransformation")
@interface Stopwatch {

    Class<? extends StopwatchHandler> value() default DefaultStopwatchHandler
}


interface StopwatchHandler {

    void handle(String method, long time)
}


final class DefaultStopwatchHandler implements StopwatchHandler {

    static final StopwatchHandler instance = new DefaultStopwatchHandler()

    void handle(String method, long time) {
        println("${method} : ${time / 1000 as long} ms")
    }
}


final class NanosecStopwatchHandler implements StopwatchHandler {

    static final StopwatchHandler instance = new NanosecStopwatchHandler()

    void handle(String method, long time) {
        println("${method} : ${time} ns")
    }
}


final class SecStopwatchHandler implements StopwatchHandler {

    static final StopwatchHandler instance = new SecStopwatchHandler()

    void handle(String method, long time) {
        println("${method} : ${time / 1000000 as long} s")
    }
}
