package com.emsoft.groovy.transform

import groovyjarjarasm.asm.Opcodes
import org.codehaus.groovy.GroovyBugError
import org.codehaus.groovy.ast.ASTNode
import org.codehaus.groovy.ast.AnnotatedNode
import org.codehaus.groovy.ast.AnnotationNode
import org.codehaus.groovy.ast.expr.ConstantExpression
import org.codehaus.groovy.ast.expr.Expression
import org.codehaus.groovy.control.SourceUnit
import org.codehaus.groovy.syntax.Token
import org.codehaus.groovy.syntax.Types
import org.codehaus.groovy.transform.ASTTransformation

abstract class BaseASTTransformation implements ASTTransformation, Opcodes {

    public static final Expression NULL_EXPR = ConstantExpression.NULL
    public static final Token ASSIGN = Token.newSymbol(Types.ASSIGN, -1, -1)
    public static final Token COMPARE_EQUAL = Token.newSymbol(Types.COMPARE_EQUAL, -1, -1)
    public static final Token COMPARE_NOT_EQUAL = Token.newSymbol(Types.COMPARE_NOT_EQUAL, -1, -1)
    public static final Token LOGICAL_AND = Token.newSymbol(Types.LOGICAL_AND, -1, -1)
    public static final Token MINUS = Token.newSymbol(Types.MINUS, -1, -1)
    public static final Token PLUS = Token.newSymbol(Types.PLUS, -1, -1)

    private SourceUnit sourceUnit

    protected void init(ASTNode[] nodes, SourceUnit sourceUnit) {
        if (nodes.length != 2 || !(nodes[0] instanceof AnnotationNode) || !(nodes[1] instanceof AnnotatedNode))
            throw new GroovyBugError("Internal error: expecting [AnnotationNode, AnnotatedNode] but got: " + Arrays.asList(nodes))
        this.sourceUnit = sourceUnit
    }

    protected void addWarning(String message) {
        sourceUnit.getErrorCollector().addWarning(1, message, sourceUnit.getCST(), sourceUnit)
    }
}
