package com.emsoft.groovy.transform

import org.codehaus.groovy.ast.AnnotationNode
import org.codehaus.groovy.ast.ClassNode
import org.codehaus.groovy.ast.expr.BinaryExpression
import org.codehaus.groovy.ast.expr.ConstantExpression
import org.codehaus.groovy.ast.expr.Expression
import org.codehaus.groovy.ast.expr.PropertyExpression
import org.codehaus.groovy.ast.stmt.ExpressionStatement
import org.codehaus.groovy.ast.stmt.Statement

class RefHelper {

    static getDefaultValue(AnnotationNode node, String member) {
        node.classNode.typeClass.getMethod(member).defaultValue
    }

    static getSimpleValue(AnnotationNode node, String member) {
        def expr = (ConstantExpression)node.getMember(member)
        expr == null ? node.classNode.typeClass.getMethod(member).defaultValue : expr.value
    }

    static getPropertyValue(AnnotationNode node, String member) {
        def expr = (PropertyExpression)node.getMember(member)
        expr == null ? node.classNode.typeClass.getMethod(member).defaultValue : expr
    }

    static Statement assignStatement(Expression fieldExpr, Expression value) {
        new ExpressionStatement(new BinaryExpression(fieldExpr, BaseASTTransformation.ASSIGN, value))
    }

    static boolean hasDeclaredMethod(ClassNode cNode, String name, int argsCount = 0) {
        def ms = cNode.getDeclaredMethods(name)
        for (m in ms) {
            def paras = m.parameters
            if (paras != null && paras.length == argsCount)
                return true
        }
        return false
    }

    static getInstancePropertyFields(ClassNode cNode) {
        cNode.properties.findAll { !it.isStatic() }
    }
}
