package com.emsoft.groovy.transform

import com.emsoft.time.Stopwatch
import org.codehaus.groovy.ast.stmt.BlockStatement
import org.codehaus.groovy.ast.stmt.ExpressionStatement
import org.codehaus.groovy.ast.stmt.TryCatchStatement
import org.codehaus.groovy.control.CompilePhase
import org.codehaus.groovy.control.SourceUnit
import org.codehaus.groovy.transform.GroovyASTTransformation
import org.codehaus.groovy.ast.*
import org.codehaus.groovy.ast.expr.*

@GroovyASTTransformation(phase = CompilePhase.CANONICALIZATION)
class StopwatchASTTransformation extends BaseASTTransformation {

    private static final def VALUE_MEMBER = "value"
    private static final def STOPWATCH_TYPE = new ClassNode(Stopwatch)
    private static final def GET_TIME_METHOD_NAME = "getElapsedTime"
    private static final def GET_INSTANCE_METHOD_NAME = "getInstance"

    void visit(ASTNode[] nodes, SourceUnit source) {
        init(nodes, source)

        def parent = nodes[1] as AnnotatedNode
        if (parent instanceof MethodNode) {
            final def annotation = nodes[0] as AnnotationNode
            final def method = parent as MethodNode

            def body = new BlockStatement()
            // объявляем и создаем переменную типа Stopwatch
            def stopwatchVar = new VariableExpression("stopwatch")
            body.addStatement(new ExpressionStatement(new DeclarationExpression(stopwatchVar, ASSIGN,
                    new ConstructorCallExpression(STOPWATCH_TYPE, ArgumentListExpression.EMPTY_ARGUMENTS))))
            // вызываем метод start()
            body.addStatement(new ExpressionStatement(new MethodCallExpression(stopwatchVar, "start",
                    ArgumentListExpression.EMPTY_ARGUMENTS)))
            // объявляем try finally
            def tryStatement = new BlockStatement()
            tryStatement.addStatements((method.code as BlockStatement).statements)
            def finallyStatement = new BlockStatement()
            // вызываем метод stop()
            finallyStatement.addStatement(new ExpressionStatement(new MethodCallExpression(stopwatchVar, "stop",
                    ArgumentListExpression.EMPTY_ARGUMENTS)))
            // вызываем метод handle(method, time) или замыкание call(method, time)
            def value = annotation.getMember(VALUE_MEMBER)
            if (value == null || value instanceof ClassExpression) {
                def handlerType = (value == null ?
                    new ClassNode(annotation.classNode.typeClass.getMethod(VALUE_MEMBER).defaultValue as Class) :
                    value.type)
                def getInstanceExpr
                if (handlerType.typeClass.methods.find { it.name == GET_INSTANCE_METHOD_NAME }) {
                    getInstanceExpr = new StaticMethodCallExpression(handlerType, GET_INSTANCE_METHOD_NAME,
                            ArgumentListExpression.EMPTY_ARGUMENTS)
                } else {
                    getInstanceExpr = new ConstructorCallExpression(handlerType,
                            ArgumentListExpression.EMPTY_ARGUMENTS)
                }
                finallyStatement.addStatement(
                        new ExpressionStatement(
                                new MethodCallExpression(
                                        getInstanceExpr,
                                        "handle",
                                        new ArgumentListExpression(
                                                new ConstantExpression(method.name),
                                                new MethodCallExpression(
                                                        stopwatchVar,
                                                        GET_TIME_METHOD_NAME,
                                                        ArgumentListExpression.EMPTY_ARGUMENTS
                                                )
                                        )
                                )
                        )
                )
            } else if (value instanceof ClosureExpression) {
                finallyStatement.addStatement(
                        new ExpressionStatement(
                                new MethodCallExpression(
                                        value,
                                        "call",
                                        new ArgumentListExpression(
                                                new ConstantExpression(method.name),
                                                new MethodCallExpression(
                                                        stopwatchVar,
                                                        GET_TIME_METHOD_NAME,
                                                        ArgumentListExpression.EMPTY_ARGUMENTS
                                                )
                                        )
                                )
                        )
                )
            }
            body.addStatement(new TryCatchStatement(tryStatement, finallyStatement))
            (method.code as BlockStatement).statements.clear()
            (method.code as BlockStatement).addStatement(body)
        }
    }
}
